unit AutoSaveExp;

////////////////////////////////////////////////////////////////////
//  
//       
//   , .. Delphi   
//   
//
//  RxLib      
//
//FREEWARE
//
// Known bugs:
// 1. When user try "Save" new project and AutoSaveExpert try do this
//    two dialog boxes show and if user click OK button then Exception will raise
//    SOLUTION: Don't try manually save new project, let AutoSaveExpert do this.
//
// (c)  , 1998
// eMail: buzuev@kbotd.ru
//
////////////////////////////////////////////////////////////////////

interface

uses Windows, ExptIntf, ToolIntf, VirtIntf, classes, Menus, SysUtils,
     EditIntf, extctrls, Dialogs, AutoSaveExpOptions, Forms,
     Controls, Registry;

type
  TvvAutoSaveExpert = class;

  ///////////////////////////////////////////////////////
  // Delphi IDE
  //      
  ///////////////////////////////////////////////////////
  TvvAutoSaveNotifier = class(TIAddInNotifier)
  public
    Expert: TvvAutoSaveExpert;  //  

    procedure FileNotification(NotifyCode: TFileNotification;
      const FileName: string; var Cancel: Boolean); override;
    procedure EventNotification(NotifyCode: TEventNotification;
      var Cancel: Boolean); override;
  end;
                
  TvvAutoSaveExpert = class(TIExpert)
  private
    FTimer: TTimer;                          //  
    FNotifier: TvvAutoSaveNotifier;          // 
    FModuleNotifierList: TList;              //     
    AutoSaveExpertMenuItem: TIMenuItemIntf;  //   Delphi IDE
                           
    procedure OnTimer(Sender: TObject);               // 
    procedure AutoSaveClick(Sender: TIMenuItemIntf);  //   
    procedure AddModuleIntf(FileName: string);        //    
  public
    function GetIDString: string; override;
    function GetStyle: TExpertStyle; override;
    function GetName: string; override;

    function GetAuthor: string; override;
    function GetComment: string; override;
    function GetPage: string; override;
    function GetMenuText: string; override;
    function GetGlyph: HICON; override;
    function GetState: TExpertState; override;
    procedure Execute; override;

    constructor Create;
    destructor Destroy; override;
  end;

  /////////////////////////////////////////////////////////////
  //  ()
  //        
  //          
  //     
  /////////////////////////////////////////////////////////////
  TvvAutoSaveModuleNotifier = class(TIModuleNotifier)
  private
    FModule: TIModuleInterface;    //   
    FModuleModified: boolean;      //   
    FExpert: TvvAutoSaveExpert;    //  
  public
    procedure Notify(NotifyCode: TNotifyCode); override;
    procedure ComponentRenamed(ComponentHandle: Pointer;
      const OldName, NewName: string); override;
  end;

//////////////////////////////////////////////////////////////////
//       IDE Delphi
//////////////////////////////////////////////////////////////////
function AddMenuItem(MenuBarName, NewMenuText, ComponentName: string; ShortCut: TShortCut;
                     Position: integer; OnClick: TIMenuClickEvent): TIMenuItemIntf;

//////////////////////////////////////////////////////////////////
//    
//////////////////////////////////////////////////////////////////
procedure HandleException;

procedure Register;

implementation

const
  SEnabledRegValueName = 'Enabled';   //   
  SDelayRegValueName =   'Delay';

procedure Register;
begin
  RegisterLibraryExpert(TvvAutoSaveExpert.Create());
end;

procedure HandleException;
begin
  if Assigned(ToolServices) then ToolServices.RaiseException(ReleaseException);
end;


//-----------------------------------------------------------
//    " Delphi 3"
//-----------------------------------------------------------
function AddMenuItem(MenuBarName, NewMenuText, ComponentName: string; ShortCut: TShortCut;
                     Position: integer; OnClick: TIMenuClickEvent): TIMenuItemIntf;
var
  MainMenu: TIMainMenuIntf;
  MEnuItem: TIMenuItemIntf;
begin
  Result := nil;

  //   
  if Assigned(ToolServices) then
    try
      MainMenu := ToolServices.GetMainMenu(); // 
      if Assigned(MainMenu) then
        try
          MenuItem := MainMenu.FindMenuItem(MenuBarName);  //    
          if Assigned(MenuItem) then
            try
              // 
              Result := MenuItem.InsertItem(Position, NewMenuText, ComponentName, '',
                        ShortCut, 0, 0, [mfEnabled, mfVisible], OnClick);
            finally
              MenuItem.Release;
            end;
        finally
          MainMenu.Release;
        end;
    except
      HandleException();
    end;
end; //AddMenuItem()



////////////////////////////////////////////////////////////////
//TvvAutoSaveExpert Methods
////////////////////////////////////////////////////////////////

function TvvAutoSaveExpert.GetIDString: string;
begin
  try
    Result := 'VV.AutoSaveExpert';
  except
    HandleException;
  end;
end;


function TvvAutoSaveExpert.GetName: string;
begin
  try
    Result := 'Auto Save Expert';
  except
    HandleException;
  end;
end;

function TvvAutoSaveExpert.GetStyle: TExpertStyle;
begin
  try
    Result := esAddin;
  except
    HandleException;
  end;
end;

constructor TvvAutoSaveExpert.Create;
var
  Registry: TRegistry;
begin
  try
    inherited Create;

    //    IDE
    AutoSaveExpertMenuItem := AddMenuItem('ToolsMenu', 'AutoSave Expert Options...', 'VVAutoSaveExpert',
                                          ShortCut(Ord('V'), [ssCtrl, ssAlt]), 1, AutoSaveClick);

    FModuleNotifierList := TList.Create();

    //  IDE
    FNotifier := TvvAutoSaveNotifier.Create();
    FNotifier.Expert := Self;
    if not ToolServices.AddNotifier(FNotifier) then raise Exception.Create('Cant Register AddIn Nofifier');

    FTimer := TTimer.Create(nil);
    FTimer.OnTimer := OnTimer;

    //  
    Registry := TRegistry.Create();
    try
      Registry.OpenKey(ToolServices.GetBaseRegistryKey() + '\VVAutoSaveExpert', true);
      try
        try
          FTimer.Interval := Registry.ReadInteger(SDelayRegValueName);
          FTimer.Enabled  := Registry.ReadBool(SEnabledRegValueName);
        except
          //  
          Registry.WriteBool(SEnabledRegValueName, true);
          Registry.WriteInteger(SDelayRegValueName, 1000*60);

          FTimer.Interval := 1000*60;
          FTimer.Enabled  := true;
        end;
      finally
        Registry.CloseKey();
      end;
    finally
      Registry.Free();
    end;

  except
    HandleException();
  end;
end;

destructor TvvAutoSaveExpert.Destroy;
var
  i: integer;
  ModuleNotifier: TvvAutoSaveModuleNotifier;
begin
  try
    //ShowMessage('Destroy');
    if Assigned(AutoSaveExpertMenuItem) then AutoSaveExpertMenuItem.Release();

    ToolServices.RemoveNotifier(FNotifier);
    FNotifier.Release();

    FTimer.Enabled := false;
    FTimer.Free();

    //  
    for i := 0 to FModuleNotifierList.Count - 1 do
    begin
      ModuleNotifier := TvvAutoSaveModuleNotifier(FModuleNotifierList.Items[i]);
      ModuleNotifier.FModule.RemoveNotifier(ModuleNotifier);
      ModuleNotifier.FModule.Release();
      ModuleNotifier.Release();
    end;
    FmoduleNotifierList.Free();

    inherited Destroy;

    //ShowMessage('xxx');
  except
    HandleException();
  end;
end;

procedure TvvAutoSaveExpert.OnTimer(Sender: TObject);
var
  i: integer;
  ModuleNotifier: TvvAutoSaveModuleNotifier;
begin
  try
    FTimer.Enabled := false;

//    ShowMessage(IntToStr(FModuleIntfList.Count));
//    ShowMessage(IntToStr(FModuleNotifierList.Count));

    //  
    for i := 0 to FModuleNotifierList.Count - 1 do
    begin
      ModuleNotifier := TvvAutoSaveModuleNotifier(FModuleNotifierList.Items[i]);
      if (ModuleNotifier.FModuleModified) then
      begin
        ModuleNotifier.FModule.Save(true);
        //ModuleNotifier.FModuleModified := false;
      end;
    end;

    FTimer.Enabled := true;
  except
    HandleException();
  end;
end;

//-------------------------------------------------------
//   
//  
//-------------------------------------------------------
procedure TvvAutoSaveExpert.AutoSaveClick(Sender: TIMenuItemIntf);
var
  Registry: TRegistry;
begin
  try
    fmAutoSaveExpertOptions := TfmAutoSaveExpertOptions.Create(nil);
    try
      fmAutoSaveExpertOptions.chbEnable.Checked := FTimer.Enabled;
      fmAutoSaveExpertOptions.eDelay.Value := FTimer.Interval / (1000*60);
      if (fmAutoSaveExpertOptions.ShowModal() = mrOk) then
      begin
        FTimer.Enabled  := fmAutoSaveExpertOptions.chbEnable.Checked;
        FTimer.Interval := trunc(fmAutoSaveExpertOptions.eDelay.Value * 1000 * 60);

        //  
        Registry := TRegistry.Create();
        try
          Registry.OpenKey(ToolServices.GetBaseRegistryKey() + '\VVAutoSaveExpert', true);
          try
            Registry.WriteBool(SEnabledRegValueName, fmAutoSaveExpertOptions.chbEnable.Checked);
            Registry.WriteInteger(SDelayRegValueName, FTimer.Interval);
          finally
            Registry.CloseKey();
          end;
        finally
          Registry.Free();
        end;

      end; //mrOk
    finally
      fmAutoSaveExpertOptions.Free();
    end;
  except
    HandleException();
  end;

end;

//------------------------------------------------
//      
//------------------------------------------------
procedure TvvAutoSaveExpert.AddModuleIntf(FileName: string);
var
  ModuleIntf: TIModuleInterface;
  ModuleNotifier: TvvAutoSaveModuleNotifier;
begin
  // 
  ModuleIntf := ToolServices.GetModuleInterface(FileName);

  if Assigned(ModuleIntf) then
  begin
    //  
    ModuleNotifier := TvvAutoSaveModuleNotifier.Create();
    ModuleNotifier.FModule := ModuleIntf;
    ModuleNotifier.FModuleModified := false;
    ModuleNotifier.FExpert := Self;

    //  
    ModuleIntf.AddNotifier(ModuleNotifier);

    //      
    //  
    FModuleNotifierList.Add(ModuleNotifier);
  end;
end; //AddModuleIntf

function TvvAutoSaveExpert.GetAuthor: string;
begin
  Result := '';
end;

function TvvAutoSaveExpert.GetComment: string;
begin
  Result := '';
end;

function TvvAutoSaveExpert.GetPage: string;
begin
  Result := '';
end;

function TvvAutoSaveExpert.GetMenuText: string;
begin
  Result := 'AutoSave Expert Options...';
end;

function TvvAutoSaveExpert.GetGlyph: HICON;
begin
  Result := 0;
end;

function TvvAutoSaveExpert.GetState: TExpertState;
begin
  Result := [];
end;

procedure TvvAutoSaveExpert.Execute;
begin
end;



///////////////////////////////////////////////////////////////////
//TAutoSaveNotifie Methods
///////////////////////////////////////////////////////////////////


//-----------------------------------------------------------
procedure TvvAutoSaveNotifier.FileNotification(NotifyCode: TFileNotification;
      const FileName: string; var Cancel: Boolean);
begin
  case NotifyCode of
    fnFileOpened,
    fnProjectOpened:
      begin
        Expert.AddModuleIntf(FileName);
      end;
  end; //case
end;

procedure TvvAutoSaveNotifier.EventNotification(NotifyCode: TEventNotification;
      var Cancel: Boolean);
begin
  try
    case NotifyCode of
      enBeforeCompile:
        begin
          //ShowMessage('BeforeCompile');
          Expert.OnTimer(Expert.FTimer);
        end;
    end; //case
  except
    HandleException();
  end;
end;

///////////////////////////////////////////////////////////////////////////
//TvvAutoSaveModuleNotifier Methods
///////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------
//          
// ..  TIAddInNotifier    
//--------------------------------------------------------------
procedure TvvAutoSaveModuleNotifier.Notify(NotifyCode: TNotifyCode);
var
 ei: TIEditorInterface;
begin
  try
    case NotifyCode of
      ncModuleDeleted:
        begin
          // 
          FModule.RemoveNotifier(Self);

          // modulrIntf
          FModule.Release();

          //     
          FExpert.FModuleNotifierList.Remove(Self);

          // 
          Release();
        end;
      ncEditorModified:
        begin
          //      
          FModuleModified := true;
        end;
      ncAfterSave:
        begin
          //   
          FModuleModified := false;
        end;
        
    end; //case
  except
    HandleException();
  end;
end;

procedure TvvAutoSaveModuleNotifier.ComponentRenamed(ComponentHandle: Pointer;
      const OldName, NewName: string);
begin
end;


end.
